==> ariadne_router-0.4.0.tar.gz: ariadne_router-0.4.0/examples/enhanced_routing_demo.py <==
#!/usr/bin/env python3
"""
Ariadne Enhanced Routing Integration Example

This example demonstrates the revolutionary intelligent routing system
that automatically optimizes quantum circuit simulation across all available backends.
"""

import time

import numpy as np
from qiskit import QuantumCircuit

# Import the new enhanced routing system
try:
    from ariadne.ml.performance_prediction import PerformancePredictor

    from ariadne.optimization.multi_objective import (
        MultiObjectiveOptimizer,
        ObjectiveWeight,
    )
    from ariadne.route.context_detection import (
        ContextDetector,
        detect_user_context,
    )
    from ariadne.route.enhanced_router import (
        EnhancedQuantumRouter,
        RouterType,
        WorkflowType,
    )
    from ariadne.router import BackendType
except ImportError as e:
    print(f"⚠️  Import error: {e}")
    print("This example requires the enhanced routing modules.")
    print("Make sure you're running from the Ariadne root directory.")
    exit(1)


def create_example_circuits():
    """Create various types of quantum circuits for demonstration."""
    circuits = {}

    # 1. Small Clifford circuit (educational)
    clifford = QuantumCircuit(3)
    clifford.h(0)
    clifford.cx(0, 1)
    clifford.cx(1, 2)
    clifford.measure_all()
    circuits["clifford"] = clifford

    # 2. Medium optimization circuit (research)
    optimization = QuantumCircuit(8)
    for i in range(8):
        optimization.ry(np.random.random() * np.pi, i)
    for i in range(7):
        optimization.cx(i, i + 1)
    for i in range(8):
        optimization.rz(np.random.random() * np.pi, i)
    optimization.measure_all()
    circuits["optimization"] = optimization

    # 3. Large random circuit (benchmarking)
    large = QuantumCircuit(15)
    np.random.seed(42)
    for _ in range(10):
        for i in range(15):
            large.ry(np.random.random() * np.pi, i)
        for i in range(0, 14, 2):
            large.cx(i, i + 1)
    large.measure_all()
    circuits["large"] = large

    return circuits


def demonstrate_context_detection():
    """Demonstrate automatic context detection."""
    print("🔍 Context Detection Demonstration")
    print("=" * 50)

    circuits = create_example_circuits()
    circuit_history = list(circuits.values())

    # Create context detector
    detector = ContextDetector()

    # Analyze user context
    context = detector.analyze_user_context(circuit_history)
    helper_context = detect_user_context(circuit_history)

    print("📊 Detected Context:")
    print(f"  Workflow Type: {context.workflow_type.value}")
    print(f"  Platform: {context.hardware_profile.platform_name}")
    print(f"  CPU Cores: {context.hardware_profile.cpu_cores}")
    print(f"  Memory: {context.hardware_profile.total_memory_gb:.1f} GB")
    print(f"  Apple Silicon: {context.hardware_profile.apple_silicon}")
    print(f"  CUDA Available: {context.hardware_profile.cuda_capable}")
    print()

    if helper_context.workflow_type == WorkflowType.BENCHMARKING:
        print("📈 Helper detected benchmarking workflow focus")
    else:
        print(f"📈 Helper detected workflow: {helper_context.workflow_type.value}")
    print()

    print("🎯 Performance Preferences:")
    prefs = context.performance_preferences
    print(f"  Speed Priority: {prefs.speed_priority:.1%}")
    print(f"  Accuracy Priority: {prefs.accuracy_priority:.1%}")
    print(f"  Memory Priority: {prefs.memory_priority:.1%}")
    print(f"  Energy Priority: {prefs.energy_priority:.1%}")
    print()

    return context


def demonstrate_enhanced_routing(context):
    """Demonstrate enhanced multi-strategy routing."""
    print("🎯 Enhanced Routing Demonstration")
    print("=" * 50)

    circuits = create_example_circuits()

    # Create enhanced router
    router = EnhancedQuantumRouter()
    router.user_context = context

    # Test different routing strategies
    strategies = [
        RouterType.SPEED_OPTIMIZER,
        RouterType.ACCURACY_OPTIMIZER,
        RouterType.HYBRID_ROUTER,
    ]

    for circuit_name, circuit in circuits.items():
        print(f"📋 Circuit: {circuit_name} ({circuit.num_qubits} qubits, depth {circuit.depth()})")

        for strategy in strategies:
            decision = router.select_optimal_backend(circuit, strategy)

            print(
                f"  {strategy.value:>15}: {decision.recommended_backend.value} "
                f"(confidence: {decision.confidence_score:.1%}, "
                f"speedup: {decision.expected_speedup:.1f}x)"
            )

        print()


def demonstrate_performance_prediction():
    """Demonstrate ML-based performance prediction."""
    print("🤖 Performance Prediction Demonstration")
    print("=" * 50)

    circuits = create_example_circuits()
    predictor = PerformancePredictor()

    backends_to_test = [BackendType.QISKIT, BackendType.STIM, BackendType.JAX_METAL]

    for circuit_name, circuit in circuits.items():
        print(f"📋 Circuit: {circuit_name}")

        for backend in backends_to_test:
            try:
                prediction = predictor.predict_performance(circuit, backend)

                print(
                    f"  {backend.value:>12}: "
                    f"Time={prediction.predicted_time:.3f}s, "
                    f"Memory={prediction.predicted_memory_mb:.0f}MB, "
                    f"Success={prediction.predicted_success_rate:.1%}"
                )

            except Exception as e:
                print(f"  {backend.value:>12}: Prediction failed - {e}")

        print()


def demonstrate_multi_objective_optimization(context):
    """Demonstrate multi-objective optimization."""
    print("⚖️  Multi-Objective Optimization Demonstration")
    print("=" * 50)

    circuits = create_example_circuits()
    optimizer = MultiObjectiveOptimizer()

    available_backends = [BackendType.QISKIT, BackendType.STIM, BackendType.JAX_METAL]

    # Test with different objective weights
    weight_configs = [
        ("Speed-Focused", ObjectiveWeight(time_weight=0.6, accuracy_weight=0.2, memory_weight=0.2)),
        (
            "Accuracy-Focused",
            ObjectiveWeight(accuracy_weight=0.6, time_weight=0.2, memory_weight=0.2),
        ),
        (
            "Balanced",
            ObjectiveWeight(time_weight=0.3, accuracy_weight=0.3, memory_weight=0.2, energy_weight=0.2),
        ),
    ]

    for circuit_name, circuit in circuits.items():
        print(f"📋 Circuit: {circuit_name}")

        for config_name, weights in weight_configs:
            try:
                result = optimizer.find_optimal_backend(circuit, available_backends, context, weights)

                print(
                    f"  {config_name:>15}: {result.backend.value} "
                    f"(score: {result.total_score:.3f}, "
                    f"rank: {result.pareto_rank})"
                )

            except Exception as e:
                print(f"  {config_name:>15}: Optimization failed - {e}")

        print()


def demonstrate_routing_explanation():
    """Demonstrate human-readable routing explanations."""
    print("💬 Routing Explanation Demonstration")
    print("=" * 50)

    circuit = create_example_circuits()["optimization"]

    router = EnhancedQuantumRouter(RouterType.HYBRID_ROUTER)

    try:
        explanation = router.explain_decision(circuit)
        print(explanation)
    except Exception as e:
        print(f"❌ Explanation failed: {e}")

        # Fallback: show basic decision
        decision = router.select_optimal_backend(circuit)
        print(f"Basic Decision: {decision.recommended_backend.value} (confidence: {decision.confidence_score:.1%})")


def run_performance_comparison():
    """Compare performance between basic and enhanced routing."""
    print("⚡ Performance Comparison")
    print("=" * 50)

    circuits = create_example_circuits()

    # Basic routing (original)
    try:
        from ariadne.router import QuantumRouter as BasicRouter

        basic_router = BasicRouter()
    except ImportError:
        print("❌ Basic router not available for comparison")
        return

    # Enhanced routing
    enhanced_router = EnhancedQuantumRouter(RouterType.HYBRID_ROUTER)

    for circuit_name, circuit in circuits.items():
        print(f"📋 Testing: {circuit_name}")

        # Time basic routing
        start_time = time.time()
        try:
            basic_decision = basic_router.select_optimal_backend(circuit)
            basic_time = time.time() - start_time
            print(f"  Basic Router: {basic_decision.recommended_backend.value} ({basic_time * 1000:.1f}ms)")
        except Exception as e:
            print(f"  Basic Router: Failed - {e}")

        # Time enhanced routing
        start_time = time.time()
        enhanced_decision = enhanced_router.select_optimal_backend(circuit)
        enhanced_time = time.time() - start_time
        print(
            f"  Enhanced Router: {enhanced_decision.recommended_backend.value} "
            f"({enhanced_time * 1000:.1f}ms, confidence: {enhanced_decision.confidence_score:.1%})"
        )

        print()


def main():
    """Run the comprehensive demonstration."""
    print("🚀 Ariadne Enhanced Routing System Demo")
    print("=" * 60)
    print("This demonstration showcases the revolutionary intelligent")
    print("routing system that automatically optimizes quantum circuit")
    print("simulation across all available backends.")
    print("=" * 60)
    print()

    try:
        # 1. Context Detection
        context = demonstrate_context_detection()

        # 2. Enhanced Routing
        demonstrate_enhanced_routing(context)

        # 3. Performance Prediction
        demonstrate_performance_prediction()

        # 4. Multi-Objective Optimization
        demonstrate_multi_objective_optimization(context)

        # 5. Routing Explanations
        demonstrate_routing_explanation()

        # 6. Performance Comparison
        run_performance_comparison()

        print("✅ Demonstration completed successfully!")
        print()
        print("🌟 Key Achievements:")
        print("  ✓ Intelligent context detection")
        print("  ✓ Multi-strategy routing optimization")
        print("  ✓ ML-based performance prediction")
        print("  ✓ Multi-objective trade-off analysis")
        print("  ✓ Human-readable decision explanations")
        print("  ✓ Real-time routing decisions (<1ms)")
        print()
        print("🎯 Ready for global quantum computing democratization!")

    except Exception as e:
        print(f"❌ Demo failed: {e}")
        print("Please ensure all modules are properly installed.")


if __name__ == "__main__":
    main()

==> ariadne_router-0.4.4.tar.gz: ariadne_router-0.4.4/examples/enhanced_routing_demo.py <==
#!/usr/bin/env python3
"""
Ariadne Enhanced Routing Integration Example

This example demonstrates the revolutionary intelligent routing system
that automatically optimizes quantum circuit simulation across all available backends.
"""

import time

import numpy as np
from qiskit import QuantumCircuit

# Import the new enhanced routing system
try:
    from ariadne.ml.performance_prediction import PerformancePredictor

    from ariadne.optimization.multi_objective import (
        MultiObjectiveOptimizer,
        ObjectiveWeight,
    )
    from ariadne.route.context_detection import (
        ContextDetector,
        detect_user_context,
    )
    from ariadne.route.enhanced_router import (
        EnhancedQuantumRouter,
        RouterType,
        WorkflowType,
    )
    from ariadne.router import BackendType
except ImportError as e:
    print(f"Warning: Import error: {e}")
    print("This example requires the enhanced routing modules.")
    print("Make sure you're running from the Ariadne root directory.")
    exit(1)


def create_example_circuits():
    """Create various types of quantum circuits for demonstration."""
    circuits = {}

    # 1. Small Clifford circuit (educational)
    clifford = QuantumCircuit(3)
    clifford.h(0)
    clifford.cx(0, 1)
    clifford.cx(1, 2)
    clifford.measure_all()
    circuits["clifford"] = clifford

    # 2. Medium optimization circuit (research)
    optimization = QuantumCircuit(8)
    for i in range(8):
        optimization.ry(np.random.random() * np.pi, i)
    for i in range(7):
        optimization.cx(i, i + 1)
    for i in range(8):
        optimization.rz(np.random.random() * np.pi, i)
    optimization.measure_all()
    circuits["optimization"] = optimization

    # 3. Large random circuit (benchmarking)
    large = QuantumCircuit(15)
    np.random.seed(42)
    for _ in range(10):
        for i in range(15):
            large.ry(np.random.random() * np.pi, i)
        for i in range(0, 14, 2):
            large.cx(i, i + 1)
    large.measure_all()
    circuits["large"] = large

    return circuits


def demonstrate_context_detection():
    """Demonstrate automatic context detection."""
    print("🔍 Context Detection Demonstration")
    print("=" * 50)

    circuits = create_example_circuits()
    circuit_history = list(circuits.values())

    # Create context detector
    detector = ContextDetector()

    # Analyze user context
    context = detector.analyze_user_context(circuit_history)
    helper_context = detect_user_context(circuit_history)

    print("Detected Context:")
    print(f"  Workflow Type: {context.workflow_type.value}")
    print(f"  Platform: {context.hardware_profile.platform_name}")
    print(f"  CPU Cores: {context.hardware_profile.cpu_cores}")
    print(f"  Memory: {context.hardware_profile.total_memory_gb:.1f} GB")
    print(f"  Apple Silicon: {context.hardware_profile.apple_silicon}")
    print(f"  CUDA Available: {context.hardware_profile.cuda_capable}")
    print()

    if helper_context.workflow_type == WorkflowType.BENCHMARKING:
        print("Helper detected benchmarking workflow focus")
    else:
        print(f"Helper detected workflow: {helper_context.workflow_type.value}")
    print()

    print("Performance Preferences:")
    prefs = context.performance_preferences
    print(f"  Speed Priority: {prefs.speed_priority:.1%}")
    print(f"  Accuracy Priority: {prefs.accuracy_priority:.1%}")
    print(f"  Memory Priority: {prefs.memory_priority:.1%}")
    print(f"  Energy Priority: {prefs.energy_priority:.1%}")
    print()

    return context


def demonstrate_enhanced_routing(context):
    """Demonstrate enhanced multi-strategy routing."""
    print("Enhanced Routing Demonstration")
    print("=" * 50)

    circuits = create_example_circuits()

    # Create enhanced router
    router = EnhancedQuantumRouter()
    router.user_context = context

    # Test different routing strategies
    strategies = [
        RouterType.SPEED_OPTIMIZER,
        RouterType.ACCURACY_OPTIMIZER,
        RouterType.HYBRID_ROUTER,
    ]

    for circuit_name, circuit in circuits.items():
        print(f"📋 Circuit: {circuit_name} ({circuit.num_qubits} qubits, depth {circuit.depth()})")

        for strategy in strategies:
            decision = router.select_optimal_backend(circuit, strategy)

            print(
                f"  {strategy.value:>15}: {decision.recommended_backend.value} "
                f"(confidence: {decision.confidence_score:.1%}, "
                f"speedup: {decision.expected_speedup:.1f}x)"
            )

        print()


def demonstrate_performance_prediction():
    """Demonstrate ML-based performance prediction."""
    print("🤖 Performance Prediction Demonstration")
    print("=" * 50)

    circuits = create_example_circuits()
    predictor = PerformancePredictor()

    backends_to_test = [BackendType.QISKIT, BackendType.STIM, BackendType.JAX_METAL]

    for circuit_name, circuit in circuits.items():
        print(f"📋 Circuit: {circuit_name}")

        for backend in backends_to_test:
            try:
                prediction = predictor.predict_performance(circuit, backend)

                print(
                    f"  {backend.value:>12}: "
                    f"Time={prediction.predicted_time:.3f}s, "
                    f"Memory={prediction.predicted_memory_mb:.0f}MB, "
                    f"Success={prediction.predicted_success_rate:.1%}"
                )

            except Exception as e:
                print(f"  {backend.value:>12}: Prediction failed - {e}")

        print()


def demonstrate_multi_objective_optimization(context):
    """Demonstrate multi-objective optimization."""
    print("⚖️  Multi-Objective Optimization Demonstration")
    print("=" * 50)

    circuits = create_example_circuits()
    optimizer = MultiObjectiveOptimizer()

    available_backends = [BackendType.QISKIT, BackendType.STIM, BackendType.JAX_METAL]

    # Test with different objective weights
    weight_configs = [
        ("Speed-Focused", ObjectiveWeight(time_weight=0.6, accuracy_weight=0.2, memory_weight=0.2)),
        (
            "Accuracy-Focused",
            ObjectiveWeight(accuracy_weight=0.6, time_weight=0.2, memory_weight=0.2),
        ),
        (
            "Balanced",
            ObjectiveWeight(time_weight=0.3, accuracy_weight=0.3, memory_weight=0.2, energy_weight=0.2),
        ),
    ]

    for circuit_name, circuit in circuits.items():
        print(f"📋 Circuit: {circuit_name}")

        for config_name, weights in weight_configs:
            try:
                result = optimizer.find_optimal_backend(circuit, available_backends, context, weights)

                print(
                    f"  {config_name:>15}: {result.backend.value} "
                    f"(score: {result.total_score:.3f}, "
                    f"rank: {result.pareto_rank})"
                )

            except Exception as e:
                print(f"  {config_name:>15}: Optimization failed - {e}")

        print()


def demonstrate_routing_explanation():
    """Demonstrate human-readable routing explanations."""
    print("💬 Routing Explanation Demonstration")
    print("=" * 50)

    circuit = create_example_circuits()["optimization"]

    router = EnhancedQuantumRouter(RouterType.HYBRID_ROUTER)

    try:
        explanation = router.explain_decision(circuit)
        print(explanation)
    except Exception as e:
        print(f"Explanation failed: {e}")

        # Fallback: show basic decision
        decision = router.select_optimal_backend(circuit)
        print(f"Basic Decision: {decision.recommended_backend.value} (confidence: {decision.confidence_score:.1%})")


def run_performance_comparison():
    """Compare performance between basic and enhanced routing."""
    print("⚡ Performance Comparison")
    print("=" * 50)

    circuits = create_example_circuits()

    # Basic routing (original)
    try:
        from ariadne.router import QuantumRouter as BasicRouter

        basic_router = BasicRouter()
    except ImportError:
        print("Basic router not available for comparison")
        return

    # Enhanced routing
    enhanced_router = EnhancedQuantumRouter(RouterType.HYBRID_ROUTER)

    for circuit_name, circuit in circuits.items():
        print(f"📋 Testing: {circuit_name}")

        # Time basic routing
        start_time = time.time()
        try:
            basic_decision = basic_router.select_optimal_backend(circuit)
            basic_time = time.time() - start_time
            print(f"  Basic Router: {basic_decision.recommended_backend.value} ({basic_time * 1000:.1f}ms)")
        except Exception as e:
            print(f"  Basic Router: Failed - {e}")

        # Time enhanced routing
        start_time = time.time()
        enhanced_decision = enhanced_router.select_optimal_backend(circuit)
        enhanced_time = time.time() - start_time
        print(
            f"  Enhanced Router: {enhanced_decision.recommended_backend.value} "
            f"({enhanced_time * 1000:.1f}ms, confidence: {enhanced_decision.confidence_score:.1%})"
        )

        print()


def main():
    """Run the comprehensive demonstration."""
    print("Ariadne Enhanced Routing System Demo")
    print("=" * 60)
    print("This demonstration showcases the revolutionary intelligent")
    print("routing system that automatically optimizes quantum circuit")
    print("simulation across all available backends.")
    print("=" * 60)
    print()

    try:
        # 1. Context Detection
        context = demonstrate_context_detection()

        # 2. Enhanced Routing
        demonstrate_enhanced_routing(context)

        # 3. Performance Prediction
        demonstrate_performance_prediction()

        # 4. Multi-Objective Optimization
        demonstrate_multi_objective_optimization(context)

        # 5. Routing Explanations
        demonstrate_routing_explanation()

        # 6. Performance Comparison
        run_performance_comparison()

        print("Demonstration completed successfully.")
        print()
        print("🌟 Key Achievements:")
        print("  ✓ Intelligent context detection")
        print("  ✓ Multi-strategy routing optimization")
        print("  ✓ ML-based performance prediction")
        print("  ✓ Multi-objective trade-off analysis")
        print("  ✓ Human-readable decision explanations")
        print("  ✓ Real-time routing decisions (<1ms)")
        print()
        print("Ready for global quantum computing democratization.")

    except Exception as e:
        print(f"Demo failed: {e}")
        print("Please ensure all modules are properly installed.")


if __name__ == "__main__":
    main()
